<?php
namespace Ems\portalGestor\Service;

use Ems\portalGestor\Service\{
    PeriodoService,
    ParamOrgService,
    OperadorService
};
use Ems\portalGestor\Util\ExpectedException;
use Ems\portalGestor\Util\HcmServices;

class ReportService {
    /** @var HcmServices */
    protected $hcmServices;

    /** @var PeriodoService */
    protected $periodoService;

    /** @var ParamOrgService */
    protected $paramOrgService;

    /** @var OperadorService */
    protected $operadorService;

    protected const RESTRICTED_REPORTS = [
        'recibo_ferias',
        'analitico_dirf',
        'informe_rendimentos',
        'demonstrativo_pagamento_via_unica',
    ];

    public function __construct(
        HcmServices $hcmServices,
        PeriodoService $periodoService,
        ParamOrgService $paramOrgService,
        OperadorService $operadorService
    ) {
        $this->hcmServices = $hcmServices;
        $this->periodoService = $periodoService;
        $this->paramOrgService = $paramOrgService;
        $this->operadorService = $operadorService;
    }

    public function generateBirtReport(
        array $row
    ): string {
        $nrOrg = (int) $row['NRORG'];
        $cdOperador = (string) $row['CDOPERADOR'];
        $hcmServicesRoute = 'generateBirtReport';

        $reportName = (string) $row['reportName'];

        $this->verifyReportIsRestricted($nrOrg, $cdOperador, $reportName);

        $nrPeriodo = $row['P_PERIODO'] ?? $row['P_NRPERIODOAPURACAO'] ?? null;
        $nrVinculo = $row['P_VINCULO'] ?? null;
        $nrTipoVinculo = $row['P_TPVINCULO'] ?? null;

        if (isset($nrPeriodo) && (isset($nrVinculo) || isset($nrTipoVinculo))) {
            $nrPeriodoEquivalente = $this->periodoService->getNrPeriodoEquivalente(
                $nrOrg, $nrVinculo, $nrPeriodo, $nrTipoVinculo
            );

            $row['P_PERIODO'] = $row['P_NRPERIODOAPURACAO'] = $nrPeriodoEquivalente;
        }

        $params = [
            'requestType' => 'Row',
            'row' => $row
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $reportLink = (string) $response['dataset'][$hcmServicesRoute]['link'];

        return $reportLink;
    }

    private function verifyReportIsRestricted(
        int $nrOrg,
        string $cdOperador,
        string $reportName
    ): void {
        $gestorCanViewReport = $this->getGestorCanViewReport($nrOrg);

        if ($gestorCanViewReport) {
            return;
        }

        $isDP = $this->operadorService->operadorIsDP($nrOrg, $cdOperador);

        if ($isDP) {
            return;
        }

        $reportIsRestricted = in_array($reportName, self::RESTRICTED_REPORTS);

        if ($reportIsRestricted) {
            throw new ExpectedException('Gestor não autorizado a prosseguir com a emissão do relatório selecionado!');
        }
    }

    public function downloadReport(
        int $nrOrg,
        string $link
    ): array {
        $hcmServicesRoute = 'downloadReport';

        $params = [
            'requestType' => 'Row',
            'row' => [
                'link' => $link
            ]
        ];

        $response = $this->hcmServices->send($hcmServicesRoute, $params, $nrOrg);
        $reportLink = (array) $response['dataset'][$hcmServicesRoute];

        return $reportLink;
    }


    public function getGestorCanViewReport(
        int $nrOrg
    ): bool {
        $paramOrg = $this->paramOrgService->get($nrOrg);
        $idRelatorioPG = $paramOrg['IDRELATORIOPG'] ?? 'S';
        $canViewReport = $idRelatorioPG === 'S';

        return $canViewReport;
    }
}
